//
// Joint Bilateral Upsampling
// Copyright (c) 2008
// University of Utah
// All rights reserved.
//
// This software is licensed under the BSD open-source license. See
// http://www.opensource.org/licenses/bsd-license.php for more detail.
//
// *************************************************************
// Redistribution and use in source and binary forms, with or 
// without modification, are permitted provided that the following 
// conditions are met:
//
// Redistributions of source code must retain the above copyright notice, 
// this list of conditions and the following disclaimer. 
//
// Redistributions in binary form must reproduce the above copyright notice, 
// this list of conditions and the following disclaimer in the documentation 
// and/or other materials provided with the distribution. 
//
// Neither the name of the University of Utah nor the names of 
// the contributors may be used to endorse or promote products derived 
// from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
// FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
// THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
// GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
// THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
// OF SUCH DAMAGE.

//-----------------------------------------------------------------------------
// Input parameters 
//-----------------------------------------------------------------------------

float screenSize : register(c29);

TEXTURE tColor;
TEXTURE tDepth;

sampler colorSampler = sampler_state
{
  Texture   = <tColor>;
  MinFilter = LINEAR;
  MagFilter = LINEAR;
  MipFilter = LINEAR;
};

sampler depthSampler = sampler_state
{
  Texture   = <tDepth>;
  MinFilter = LINEAR;
  MagFilter = LINEAR;
  MipFilter = LINEAR;
};


//-----------------------------------------------------------------------------
// Vertex Shader INPUTS/OUTPUTS
//-----------------------------------------------------------------------------
struct BILATERAL_VS_INPUT
{
float4 position : POSITION;
float4 color : COLOR;
float4 texCoord0 : TEXCOORD0;
float4 texCoord1 : TEXCOORD1;
float4 texCoord2 : TEXCOORD2;
};

struct BILATERAL_VS_OUTPUT
{
float4 position : POSITION;
float4 color : COLOR;
float4 texCoord0 : TEXCOORD0;
float4 texCoord1 : TEXCOORD1;
float4 texCoord2 : TEXCOORD2;
};

//-----------------------------------------------------------------------------
// Pixel Shader INPUTS/OUTPUTS
//-----------------------------------------------------------------------------
struct BILATERAL_PS_INPUT
{
float4 color : COLOR;
float4 texCoord0 : TEXCOORD0;
float4 texCoord1 : TEXCOORD1;
float4 texCoord2	: TEXCOORD2;
};

struct BILATERAL_PS_OUTPUT
{
float4 color : COLOR;
};

//-----------------------------------------------------------------------------
// Bilateral Filter
//-----------------------------------------------------------------------------
BILATERAL_VS_OUTPUT BilateralFilter_VS(Test_VS_INPUT IN)
{
  BILATERAL_VS_OUTPUT OUT = (BILATERAL_VS_OUTPUT) 0;
  
  OUT.position  = IN.position;
  OUT.texCoord0 = IN.texCoord0;
  OUT.texCoord1 = IN.texCoord1;
  OUT.texCoord2 = IN.texCoord2;

  return OUT;
}

//-----------------------------------------------------------------------------
// Bilateral Filter
//-----------------------------------------------------------------------------
BILATERAL_PS_OUTPUT BilateralFilter_PS(BILATERAL_PS_INPUT IN, uniform float screenSize)
{
  BILATERAL_PS_OUTPUT OUT = (BILATERAL_PS_OUTPUT) 0;

  float curr_depth = tex2D(depthSampler, float2(IN.texCoord0.xy)).x;
  if (curr_depth < 1.0) discard;
  float4 sum = float4(0.0, 0.0, 0.0, 0.0);
  float4 sample;
  float radius = 2.0;
  float2 pos;
  float i,j;
  float sigma = 2.0*radius*radius;
  float domain_gaussian = 0.0;
  float weight = 0.0;
  float distance = 0.0;
  float sample_depth;
  float range_gaussian;
  float sigma2 = 0.01;
	
  for (i = -radius; i <= radius; i+=1.0)
  {
    for (j = -radius; j <= radius; j+=1.0)
    {
      float dx = i/screenSize;
      float dy = j/screenSize;
      pos.x = IN.texCoord0.x+dx;
      pos.y = IN.texCoord0.y+dy;
      sample = tex2D(colorSampler, pos);
      distance = i*i+j*j;
      domain_gaussian = exp(-distance/sigma);
			
      sample_depth = tex2D(depthSampler, pos).x;
      distance = (curr_depth-sample_depth)*(curr_depth-sample_depth);
      range_gaussian = exp(-distance/sigma2);
			
      sum+=sample*domain_gaussian*range_gaussian;
      weight+=domain_gaussian*range_gaussian;
    }
  }


  OUT.color = sum/weight;
  return OUT;
}

//-----------------------------------------------------------------------------
// technique BilateralFilter
//-----------------------------------------------------------------------------
technique BilateralFilter {

  pass p0 {
    CullMode = None;
    FillMode = Solid;
    ShadeMode = Flat;
    AlphaBlendEnable = True;
    PixelShaderConstant[29] = <screenSize>;
    VertexShader = compile VS_COMPILER_TARGET BilateralFilter_VS();
    PixelShader  = compile PS_COMPILER_TARGET BilateralFilter_PS(SCREEN_DIMENSION);
  }
}

